@@ -1,5 +1,5 @@
 /*  Audacious - Cross-platform multimedia player
- *  Copyright (C) 2005-2008  Audacious team
+ *  Copyright (C) 2005-2009  Audacious team
  *
  *  Based on BMP:
  *  Copyright (C) 2003-2004  BMP development team.
@@ -35,8 +35,6 @@
 #include "main.h"
 #include "input.h"
 #include "playback.h"
-
-#include "playlist.h"
 #include "configdb.h"
 
 #include "flow.h"
@@ -44,8 +42,8 @@
 #include "pluginenum.h"
 
 #include "effect.h"
+#include "vis_runner.h"
 #include "volumecontrol.h"
-#include "visualization.h"
 
 #include "libSAD.h"
 #include "util.h"
@@ -57,6 +55,9 @@
 # include "src_flow.h"
 #endif
 
+#define IS_S16_NE(a) ((a) == FMT_S16_NE || (G_BYTE_ORDER == G_LITTLE_ENDIAN && \
+ (a) == FMT_S16_LE) || (G_BYTE_ORDER == G_BIG_ENDIAN && (a) == FMT_S16_BE))
+
 OutputPluginData op_data = {
     NULL,
     NULL
@@ -97,58 +98,29 @@ get_current_output_plugin(void)
     return op_data.current_output_plugin;
 }
 
-void
-set_current_output_plugin(gint i)
+void set_current_output_plugin (gint i)
 {
-    gboolean playing;
-    OutputPlugin *op = get_current_output_plugin();
+    gboolean playing = playback_get_playing ();
+    gboolean paused;
+    gint time;
 
-    GList *node = g_list_nth(op_data.output_list, i);
-    if (!node) {
-        op_data.current_output_plugin = NULL;
-        return;
+    if (playing)
+    {
+        paused = playback_get_paused ();
+        time = playback_get_time ();
+        playback_stop ();
     }
 
-    op_data.current_output_plugin = node->data;
+    op_data.current_output_plugin = g_list_nth(op_data.output_list, i)->data;
 
-    playing = playback_get_playing();
-
-    if (playing == TRUE)
+    if (playing)
     {
-        guint time, pos;
-        PlaylistEntry *entry;
+        playback_initiate ();
 
-	plugin_set_current((Plugin *)op);
+        if (paused)
+            playback_pause ();
 
-        /* don't stop yet, get the seek time and playlist position first */
-        pos = playlist_get_position(playlist_get_active());
-        time = op->output_time();
-
-        /* reset the audio system */
-        playback_stop();
-        op->close_audio();
-
-        g_usleep(300000);
-
-        /* wait for the playback thread to come online */
-        while (playback_get_playing())
-            g_message("waiting for audio system shutdown...");
-
-        /* wait for the playback thread to come online */
-        playlist_set_position(playlist_get_active(), pos);
-        entry = playlist_get_entry_to_play(playlist_get_active());
-        playback_play_file(entry);
-
-        while (!playback_get_playing())
-        {
-            gtk_main_iteration();
-                g_message("waiting for audio system startup...");
-        }
-
-        /* and signal a reseek */
-        if (playlist_get_current_length(playlist_get_active()) > -1 &&
-            time <= (playlist_get_current_length(playlist_get_active())))
-            playback_seek(time / 1000);
+        playback_seek (time);
     }
 }
 
@@ -183,39 +155,32 @@ output_configure(gint i)
 void
 output_get_volume(gint * l, gint * r)
 {
-    *l = *r = -1;
-
-    if (!op_data.current_output_plugin)
-        return;
-
-    if (!op_data.current_output_plugin->get_volume)
-        return;
-
     if (cfg.software_volume_control)
-        volumecontrol_get_volume_state(l, r);
+    {
+        * l = cfg.sw_volume_left;
+        * r = cfg.sw_volume_right;
+    }
+    else if (op_data.current_output_plugin &&
+     op_data.current_output_plugin->get_volume)
+        op_data.current_output_plugin->get_volume (l, r);
     else
     {
-        plugin_set_current((Plugin *)op_data.current_output_plugin);
-        op_data.current_output_plugin->get_volume(l, r);
+        * l = -1;
+        * r = -1;
     }
 }
 
 void
 output_set_volume(gint l, gint r)
 {
-    if (!op_data.current_output_plugin)
-        return;
-
-    if (!op_data.current_output_plugin->set_volume)
-        return;
-
     if (cfg.software_volume_control)
-        volumecontrol_set_volume_state(l, r);
-    else
     {
-        plugin_set_current((Plugin *)op_data.current_output_plugin);
-        op_data.current_output_plugin->set_volume(l, r);
+        cfg.sw_volume_left = l;
+        cfg.sw_volume_right = r;
     }
+    else if (op_data.current_output_plugin &&
+     op_data.current_output_plugin->set_volume)
+        op_data.current_output_plugin->set_volume (l, r);
 }
 
 void
@@ -232,9 +197,10 @@ gint
 get_written_time(void)
 {
     OutputPlugin *op = get_current_output_plugin();
+    InputPlayback *ip = get_current_input_playback();
 
     plugin_set_current((Plugin *)op);
-    return op->written_time();
+    return op->written_time() - ip->start;
 }
 
 /* called by input plugin to peek at the output plugin's output progress */
@@ -249,10 +215,7 @@ get_output_time(void)
 
 static SAD_dither_t *sad_state_to_float = NULL;
 static SAD_dither_t *sad_state_from_float = NULL;
-static float *sad_float_buf = NULL;
-static void *sad_out_buf = NULL;
-static gsize sad_float_buf_length = 0;
-static gsize sad_out_buf_length = 0;
+
 static ReplayGainInfo replay_gain_info = {
     .track_gain = 0.0,
     .track_peak = 0.0,
@@ -342,7 +305,24 @@ output_open_audio(AFormat fmt, gint rate, gint nch)
         bit_depth = cfg.output_bit_depth;
 
         AUDDBG("bit depth: %d\n", bit_depth);
-        output_fmt = (bit_depth == 24) ? FMT_S24_NE : FMT_S16_NE;
+        switch (bit_depth) {
+            case 32:
+                output_fmt = FMT_S32_NE;
+                break;
+            case 24:
+                output_fmt = FMT_S24_NE;
+                break;
+            case 16:
+                output_fmt = FMT_S16_NE;
+                break;
+            case 0:
+                output_fmt = FMT_FLOAT;
+                break;
+            default:
+                AUDDBG("unsupported bit depth requested %d\n", bit_depth);
+                output_fmt = FMT_S16_NE;
+                break;
+        }
 
         freeSAD();
 
@@ -421,6 +401,7 @@ output_close_audio(void)
 {
     OutputPlugin *op = get_current_output_plugin();
 
+    vis_runner_flush ();
     freeSAD();
 
     AUDDBG("clearing RG settings\n");
@@ -432,17 +413,6 @@ output_close_audio(void)
 #ifdef USE_SAMPLERATE
     src_flow_free();
 #endif
-    /* Do not close if there are still songs to play and the user has
-     * not requested a stop.  --nenolod
-     */
-    Playlist *pl = playlist_get_active();
-    if (ip_data.stop == FALSE &&
-       (playlist_get_position_nolock(pl) < playlist_get_length(pl) - 1) &&
-       !cfg.stopaftersong &&
-       !(cfg.no_playlist_advance && !cfg.repeat)) {
-            AUDDBG("leaving audio opened\n");
-            return;
-        }
 
     /* Sanity check. */
     if (op == NULL)
@@ -462,6 +432,8 @@ output_flush(gint time)
 {
     OutputPlugin *op = get_current_output_plugin();
 
+    vis_runner_flush ();
+
     if (op == NULL)
         return;
 
@@ -505,136 +477,118 @@ output_buffer_playing(void)
     return op->buffer_playing();
 }
 
-/* called by input plugin when data is ready */
-void
-output_pass_audio(InputPlayback *playback,
-              AFormat fmt,       /* output format        */
-              gint nch,          /* channels             */
-              gint length,       /* length of sample     */
-              gpointer ptr,      /* data                 */
-              int *going         /* 0 when time to stop  */
-              )
+static Flow * get_postproc_flow (void)
 {
-    static Flow *visualization_flow = NULL;
-    static Flow *postproc_flow = NULL;
-    static Flow *legacy_flow = NULL;
-    OutputPlugin *op = playback->output;
-    gint writeoffs;
-    gpointer float_ptr;
+    static Flow * flow = NULL;
 
-    if (visualization_flow == NULL)
+    if (flow == NULL)
     {
-        visualization_flow = flow_new();
-        flow_link_element(visualization_flow, vis_flow);
+        flow = flow_new ();
+
+        #ifdef USE_SAMPLERATE
+            flow_link_element (flow, src_flow);
+        #endif
+
+        flow_link_element (flow, equalizer_flow);
+        flow_link_element (flow, volumecontrol_flow);
     }
 
-    plugin_set_current((Plugin *)(playback->output));
-    gint time = playback->output->written_time();
+    return flow;
+}
 
-    flow_execute(visualization_flow, time, &ptr, length, fmt, decoder_srate, nch);
+static Flow * get_legacy_flow (void)
+{
+    static Flow * flow = NULL;
 
-    if (!bypass_dsp) {
-
-        if(length <= 0 || sad_state_from_float == NULL || sad_state_to_float == NULL) return;
-
-        if (legacy_flow == NULL)
-        {
-            legacy_flow = flow_new();
-            flow_link_element(legacy_flow, effect_flow);
-        }
-
-        if (postproc_flow == NULL)
-        {
-            postproc_flow = flow_new();
-#ifdef USE_SAMPLERATE
-            flow_link_element(postproc_flow, src_flow);
-#endif
-            flow_link_element(postproc_flow, equalizer_flow);
-            flow_link_element(postproc_flow, volumecontrol_flow);
-        }
-
-        int frames = length / nch / FMT_SIZEOF(fmt);
-        int len = frames * nch * sizeof(float);
-        if(sad_float_buf == NULL || sad_float_buf_length < len) {
-            sad_float_buf_length = len;
-            sad_float_buf = smart_realloc(sad_float_buf, &sad_float_buf_length);
-        }
-
-        SAD_dither_process_buffer(sad_state_to_float, ptr, sad_float_buf, frames);
-        float_ptr = sad_float_buf;
-
-        length = flow_execute(postproc_flow,
-                              time,
-                              &float_ptr,
-                              len,
-                              FMT_FLOAT,
-                              decoder_srate,
-                              nch);
-
-        frames = length / nch / sizeof(float);
-        len = frames * nch * FMT_SIZEOF(op_state.fmt);
-        if(sad_out_buf == NULL || sad_out_buf_length < len) {
-            sad_out_buf_length = len;
-            sad_out_buf = smart_realloc(sad_out_buf, &sad_out_buf_length);
-        }
-
-        SAD_dither_process_buffer(sad_state_from_float, float_ptr, sad_out_buf, frames);
-
-        length = len;
-        ptr = sad_out_buf;
-
-        if (op_state.fmt == FMT_S16_NE || (op_state.fmt == FMT_S16_LE && G_BYTE_ORDER == G_LITTLE_ENDIAN) ||
-                                          (op_state.fmt == FMT_S16_BE && G_BYTE_ORDER == G_BIG_ENDIAN)) {
-            length = flow_execute(legacy_flow, time, &ptr, length, op_state.fmt, op_state.rate, op_state.nch);
-        } else {
-            AUDDBG("legacy_flow can deal only with S16_NE streams\n"); /*FIXME*/
-        }
-    } /* !bypass_dsp */
-
-    /**** write it out ****/
-
-    writeoffs = 0;
-    while (writeoffs < length)
+    if (flow == NULL)
     {
-        int writable = length - writeoffs;
+        flow = flow_new ();
+        flow_link_element (flow, effect_flow);
+    }
 
-        if (writable > 2048)
-            writable = 2048;
+    return flow;
+}
 
-        if (writable == 0)
-            return;
+void output_pass_audio (InputPlayback * playback, AFormat fmt, gint channels,
+ gint size, void * data, gint * going)
+{
+    gint samples = size / FMT_SIZEOF (fmt);
+    void * allocated = NULL;
+    gint time = playback->output->written_time ();
 
-        while (op->buffer_free() < writable)   /* wait output buf */
+    if (! bypass_dsp)
+    {
+        if (fmt != FMT_FLOAT)
         {
-            GTimeVal pb_abs_time;
+            gfloat * new = g_malloc (sizeof (gfloat) * samples);
 
-            g_get_current_time(&pb_abs_time);
-            g_time_val_add(&pb_abs_time, 10000);
+            if (IS_S16_NE (fmt))
+                s16_to_float (data, new, samples);
+            else
+                SAD_dither_process_buffer (sad_state_to_float, data, new,
+                 samples / channels);
 
-            if (going && !*going)              /* thread stopped? */
-                return;                        /* so finish */
-
-            if (ip_data.stop)                  /* has a stop been requested? */
-                return;                        /* yes, so finish */
-
-            /* else sleep for retry */
-            g_mutex_lock(playback->pb_change_mutex);
-            g_cond_timed_wait(playback->pb_change_cond, playback->pb_change_mutex, &pb_abs_time);
-            g_mutex_unlock(playback->pb_change_mutex);
+            data = new;
+            g_free (allocated);
+            allocated = new;
         }
 
-        if (ip_data.stop)
-            return;
+        vis_runner_pass_audio (time, data, samples, channels);
 
-        if (going && !*going)                  /* thread stopped? */
-            return;                            /* so finish */
+        samples = flow_execute (get_postproc_flow (), time, & data, sizeof
+         (gfloat) * samples, FMT_FLOAT, decoder_srate, channels) / sizeof
+         (gfloat);
 
-        /* do output */
-	plugin_set_current((Plugin *)op);
-        op->write_audio(((guint8 *) ptr) + writeoffs, writable);
+        if (data != allocated)
+        {
+            g_free (allocated);
+            allocated = NULL;
+        }
 
-        writeoffs += writable;
+        if (op_state.fmt != FMT_FLOAT)
+        {
+            void * new = g_malloc (FMT_SIZEOF (op_state.fmt) * samples);
+
+            if (cfg.no_dithering && IS_S16_NE (op_state.fmt))
+                float_to_s16 (data, new, samples);
+            else
+                SAD_dither_process_buffer (sad_state_from_float, data, new,
+                 samples / channels);
+
+            data = new;
+            g_free (allocated);
+            allocated = new;
+        }
+
+        if (IS_S16_NE (op_state.fmt))
+        {
+            samples = flow_execute (get_legacy_flow (), time, & data, 2 *
+             samples, op_state.fmt, op_state.rate, op_state.nch) / 2;
+
+            if (data != allocated)
+            {
+                g_free (allocated);
+                allocated = NULL;
+            }
+        }
     }
+
+    for (;;)
+    {
+        gint ready = playback->output->buffer_free () / FMT_SIZEOF (op_state.fmt);
+
+        ready = MIN (ready, samples);
+        playback->output->write_audio (data, FMT_SIZEOF (op_state.fmt) * ready);
+        data = (char *) data + FMT_SIZEOF (op_state.fmt) * ready;
+        samples -= ready;
+
+        if (! samples)
+            break;
+
+        g_usleep (50000);
+    }
+
+    g_free (allocated);
 }
 
 /* called by input plugin when RG info available --asphyx */
