@@ -1,5 +1,5 @@
 /*  Audacious - Cross-platform multimedia player
- *  Copyright (C) 2005-2007  Audacious development team
+ *  Copyright (C) 2005-2009  Audacious development team
  *
  *  Based on BMP:
  *  Copyright (C) 2003-2004  BMP development team
@@ -39,14 +39,17 @@
 
 #include "pluginenum.h"
 
-#include "discovery.h"
+#include "audconfig.h"
+#include "credits.h"
 #include "effect.h"
 #include "general.h"
 #include "input.h"
 #include "main.h"
+#include "chardet.h"
 #include "output.h"
 #include "playback.h"
-#include "playlist.h"
+#include "playlist-new.h"
+#include "playlist-utils.h"
 #include "audstrings.h"
 #include "util.h"
 #include "visualization.h"
@@ -111,6 +114,13 @@ static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
     .vfs_fget_be32 = vfs_fget_be32,
     .vfs_fget_be64 = vfs_fget_be64,
 
+    .vfs_fput_le16 = vfs_fput_le16,
+    .vfs_fput_le32 = vfs_fput_le32,
+    .vfs_fput_le64 = vfs_fput_le64,
+    .vfs_fput_be16 = vfs_fput_be16,
+    .vfs_fput_be32 = vfs_fput_be32,
+    .vfs_fput_be64 = vfs_fput_be64,
+
     .cfg_db_open = cfg_db_open,
     .cfg_db_close = cfg_db_close,
 
@@ -128,23 +138,6 @@ static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
 
     .cfg_db_unset_key = cfg_db_unset_key,
 
-    .tuple_new = tuple_new,
-    .tuple_new_from_filename = tuple_new_from_filename,
-    .tuple_associate_string = tuple_associate_string,
-    .tuple_associate_int = tuple_associate_int,
-    .tuple_disassociate = tuple_disassociate,
-    .tuple_get_value_type = tuple_get_value_type,
-    .tuple_get_string = tuple_get_string,
-    .tuple_get_int = tuple_get_int,
-
-    .tuple_formatter_process_string = tuple_formatter_process_string,
-    .tuple_formatter_process_function = tuple_formatter_process_function,
-    .tuple_formatter_process_construct = tuple_formatter_process_construct,
-    .tuple_formatter_process_expr = tuple_formatter_process_expr,
-    .tuple_formatter_register_function = tuple_formatter_register_function,
-    .tuple_formatter_register_expression = tuple_formatter_register_expression,
-    .tuple_formatter_make_title_string = tuple_formatter_make_title_string,
-
     .mime_get_plugin = mime_get_plugin,
     .mime_set_plugin = mime_set_plugin,
 
@@ -152,24 +145,14 @@ static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
     .uri_set_plugin = uri_set_plugin,
 
     .util_info_dialog = util_info_dialog,
-    .get_gentitle_format = get_gentitle_format,
 
     .smart_realloc = smart_realloc,
     .sadfmt_from_afmt = sadfmt_from_afmt,
 
-    .escape_shell_chars = escape_shell_chars,
-    .str_append = str_append,
-    .str_replace = str_replace,
-    .str_replace_in = str_replace_in,
-    .str_has_prefix_nocase = str_has_prefix_nocase,
-    .str_has_suffix_nocase = str_has_suffix_nocase,
-    .str_has_suffixes_nocase = str_has_suffixes_nocase,
-    .str_to_utf8_fallback = str_to_utf8_fallback,
-    .str_to_utf8 = str_to_utf8,
-    .filename_to_utf8 = filename_to_utf8,
-    .str_skip_chars = str_skip_chars,
-    .convert_title_text = convert_title_text,
-    .chardet_to_utf8 = chardet_to_utf8,
+    .util_add_url_history_entry = util_add_url_history_entry,
+
+    .str_to_utf8 = cd_str_to_utf8,
+    .chardet_to_utf8 = cd_chardet_to_utf8,
 
     .playlist_container_register = playlist_container_register,
     .playlist_container_unregister = playlist_container_unregister,
@@ -177,114 +160,90 @@ static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
     .playlist_container_write = playlist_container_write,
     .playlist_container_find = playlist_container_find,
 
-    .playlist_entry_new = playlist_entry_new,
-    .playlist_entry_free = playlist_entry_free,
-
-    .playlist_add_playlist = playlist_add_playlist,
-    .playlist_remove_playlist = playlist_remove_playlist,
-    .playlist_select_playlist = playlist_select_playlist,
-    .playlist_select_next = playlist_select_next,
-    .playlist_select_prev = playlist_select_prev,
-    .playlist_get_playlists = playlist_get_playlists,
-
-    .playlist_clear_only = playlist_clear_only,
-    .playlist_clear = playlist_clear,
+    .playlist_count = playlist_count,
+    .playlist_insert = playlist_insert,
     .playlist_delete = playlist_delete,
 
-    .playlist_add = playlist_add,
-    .playlist_ins = playlist_ins,
-    .playlist_add_dir = playlist_add_dir,
-    .playlist_ins_dir = playlist_ins_dir,
-    .playlist_add_url = playlist_add_url,
-    .playlist_ins_url = playlist_ins_url,
-
-    .playlist_next = playlist_next,
-    .playlist_prev = playlist_prev,
-
-    .playlist_queue = playlist_queue,
-    .playlist_queue_position = playlist_queue_position,
-    .playlist_queue_remove = playlist_queue_remove,
-    .playlist_queue_get_length = playlist_queue_get_length,
-    .playlist_is_position_queued = playlist_is_position_queued,
-    .playlist_clear_queue = playlist_clear_queue,
-    .playlist_get_queue_position = playlist_get_queue_position,
-    .playlist_get_queue_position_number = playlist_get_queue_position_number,
-    .playlist_get_queue_qposition_number = playlist_get_queue_qposition_number,
-
-    .playlist_eof_reached = playlist_eof_reached,
-    .playlist_set_position = playlist_set_position,
-
-    .playlist_get_length = playlist_get_length,
-    .playlist_get_position = playlist_get_position,
-    .playlist_get_position_nolock = playlist_get_position_nolock,
-    .playlist_get_info_text = playlist_get_info_text,
-    .playlist_get_current_length = playlist_get_current_length,
-
-    .playlist_save = playlist_save,
-    .playlist_load = playlist_load,
-
-    .playlist_sort = playlist_sort,
-    .playlist_sort_selected = playlist_sort_selected,
-
-    .playlist_reverse = playlist_reverse,
-    .playlist_random = playlist_random,
-    .playlist_remove_duplicates = playlist_remove_duplicates,
-    .playlist_remove_dead_files = playlist_remove_dead_files,
-
-    .playlist_fileinfo_current = ui_fileinfo_show_current,
-    .playlist_fileinfo = ui_fileinfo_show,
-
-    .playlist_delete_index = playlist_delete_index,
-
-    .playlist_get_entry_to_play = playlist_get_entry_to_play,
-
+    .playlist_set_filename = playlist_set_filename,
     .playlist_get_filename = playlist_get_filename,
-    .playlist_get_songtitle = playlist_get_songtitle,
-    .playlist_get_tuple = playlist_get_tuple,
-    .playlist_get_songtime = playlist_get_songtime,
+    .playlist_set_title = playlist_set_title,
+    .playlist_get_title = playlist_get_title,
 
-    .playlist_get_selected = playlist_get_selected,
-    .playlist_get_num_selected = playlist_get_num_selected,
+    .playlist_set_active = playlist_set_active,
+    .playlist_get_active = playlist_get_active,
+    .playlist_set_playing = playlist_set_playing,
+    .playlist_get_playing = playlist_get_playing,
 
-    .playlist_get_total_time = playlist_get_total_time,
-    .playlist_select_search = playlist_select_search,
+    .playlist_entry_count = playlist_entry_count,
+    .playlist_entry_insert = playlist_entry_insert,
+    .playlist_entry_insert_batch = playlist_entry_insert_batch,
+    .playlist_entry_delete = playlist_entry_delete,
+
+    .playlist_entry_get_filename = playlist_entry_get_filename,
+    .playlist_entry_get_decoder = playlist_entry_get_decoder,
+    .playlist_entry_set_tuple = playlist_entry_set_tuple,
+    .playlist_entry_get_tuple = playlist_entry_get_tuple,
+    .playlist_entry_get_title = playlist_entry_get_title,
+    .playlist_entry_get_length = playlist_entry_get_length,
+
+    .playlist_set_position = playlist_set_position,
+    .playlist_get_position = playlist_get_position,
+
+    .playlist_entry_set_selected = playlist_entry_set_selected,
+    .playlist_entry_get_selected = playlist_entry_get_selected,
+    .playlist_selected_count = playlist_selected_count,
     .playlist_select_all = playlist_select_all,
-    .playlist_select_range = playlist_select_range,
-    .playlist_select_invert_all = playlist_select_invert_all,
-    .playlist_select_invert = playlist_select_invert,
 
-    .playlist_read_info_selection = playlist_read_info_selection,
-    .playlist_read_info = playlist_read_info,
+    .playlist_shift = playlist_shift,
+    .playlist_shift_selected = playlist_shift_selected,
+    .playlist_delete_selected = playlist_delete_selected,
+    .playlist_reverse = playlist_reverse,
+    .playlist_randomize = playlist_randomize,
+
+    .playlist_sort_by_filename = playlist_sort_by_filename,
+    .playlist_sort_by_tuple = playlist_sort_by_tuple,
+    .playlist_sort_selected_by_filename = playlist_sort_selected_by_filename,
+    .playlist_sort_selected_by_tuple = playlist_sort_selected_by_tuple,
+
+    .playlist_rescan = playlist_rescan,
+
+    .playlist_get_total_length = playlist_get_total_length,
+    .playlist_get_selected_length = playlist_get_selected_length,
 
     .playlist_set_shuffle = playlist_set_shuffle,
 
-    .playlist_clear_selected = playlist_clear_selected,
+    .playlist_queue_count = playlist_queue_count,
+    .playlist_queue_insert = playlist_queue_insert,
+    .playlist_queue_insert_selected = playlist_queue_insert_selected,
+    .playlist_queue_get_entry = playlist_queue_get_entry,
+    .playlist_queue_find_entry = playlist_queue_find_entry,
+    .playlist_queue_delete = playlist_queue_delete,
 
-    .get_playlist_nth = get_playlist_nth,
+    .playlist_prev_song = playlist_prev_song,
+    .playlist_next_song = playlist_next_song,
 
-    .playlist_set_current_name = playlist_set_current_name,
-    .playlist_get_current_name = playlist_get_current_name,
+    .playlist_update_pending = playlist_update_pending,
 
-    .playlist_filename_set = playlist_filename_set,
-    .playlist_filename_get = playlist_filename_get,
+    .get_gentitle_format = get_gentitle_format,
 
-    .playlist_new = playlist_new,
-    .playlist_free = playlist_free,
-    .playlist_new_from_selected = playlist_new_from_selected,
+    .playlist_sort_by_scheme = playlist_sort_by_scheme,
+    .playlist_sort_selected_by_scheme = playlist_sort_selected_by_scheme,
+    .playlist_remove_duplicates_by_scheme = playlist_remove_duplicates_by_scheme,
+    .playlist_remove_failed = playlist_remove_failed,
+    .playlist_select_by_patterns = playlist_select_by_patterns,
 
-    .is_playlist_name = is_playlist_name,
+    .filename_is_playlist = filename_is_playlist,
 
-    .playlist_load_ins_file = playlist_load_ins_file,
-    .playlist_load_ins_file_tuple = playlist_load_ins_file_tuple,
-
-    .playlist_get_active = playlist_get_active,
-    .playlist_playlists_equal = playlist_playlists_equal,
+    .playlist_insert_playlist = playlist_insert_playlist,
+    .playlist_save = playlist_save,
+    .playlist_add_folder = playlist_add_folder,
 
     .ip_state = &ip_data,
     ._cfg = &cfg,
 
     .hook_associate = hook_associate,
     .hook_dissociate = hook_dissociate,
+    .hook_dissociate_full = hook_dissociate_full,
     .hook_register = hook_register,
     .hook_call = hook_call,
 
@@ -314,10 +273,10 @@ static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
     .drct_get_playing = drct_get_playing,
     .drct_get_paused = drct_get_paused,
     .drct_get_stopped = drct_get_stopped,
-    .drct_get_info = drct_get_info,
-    .drct_get_time = drct_get_time,
-    .drct_get_length = drct_get_length,
-    .drct_seek = drct_seek,
+    .drct_get_info = playback_get_info,
+    .drct_get_time = playback_get_time,
+    .drct_get_length = playback_get_length,
+    .drct_seek = playback_seek,
     .drct_get_volume = drct_get_volume,
     .drct_set_volume = drct_set_volume,
     .drct_get_volume_main = drct_get_volume_main,
@@ -335,6 +294,8 @@ static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
     .drct_pl_get_time = drct_pl_get_time,
     .drct_pl_get_pos = drct_pl_get_pos,
     .drct_pl_get_file = drct_pl_get_file,
+    .drct_pl_open = drct_pl_open,
+    .drct_pl_open_list = drct_pl_open_list,
     .drct_pl_add = drct_pl_add,
     .drct_pl_clear = drct_pl_clear,
     .drct_pl_get_length = drct_pl_get_length,
@@ -352,9 +313,6 @@ static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
     .drct_pq_get_position = drct_pq_get_position,
     .drct_pq_get_queue_position = drct_pq_get_queue_position,
 
-    .prefswin_page_new = prefswin_page_new,
-    .prefswin_page_destroy = prefswin_page_destroy,
-
     .fileinfopopup_create = fileinfopopup_create,
     .fileinfopopup_destroy = fileinfopopup_destroy,
     .fileinfopopup_show_from_title = fileinfopopup_show_from_title,
@@ -383,6 +341,7 @@ static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
     .set_pvt_data = set_pvt_data,
 
     .event_queue = event_queue,
+    .event_queue_with_data_free = event_queue_with_data_free,
 
     .calc_mono_freq = calc_mono_freq,
     .calc_mono_pcm = calc_mono_pcm,
@@ -399,7 +358,23 @@ static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
     .output_plugin_cleanup = output_plugin_cleanup,
     .output_plugin_reinit = output_plugin_reinit,
 
-    .get_plugin_menu = get_plugin_menu
+    .get_plugin_menu = get_plugin_menu,
+    .playback_get_title = playback_get_title,
+    .fileinfo_show = ui_fileinfo_show,
+    .fileinfo_show_current = ui_fileinfo_show_current,
+    .save_all_playlists = save_all_playlists,
+
+    .interface_get_current = interface_get_current,
+    .interface_toggle_visibility = interface_toggle_visibility,
+    .interface_show_error = interface_show_error_message,
+
+    .get_audacious_credits = get_audacious_credits,
+
+    /* playlist segmentation -- added in Audacious 2.2. */
+    .playlist_entry_set_segmentation = playlist_entry_set_segmentation,
+    .playlist_entry_is_segmented = playlist_entry_is_segmented,
+    .playlist_entry_get_start_time = playlist_entry_get_start_time,
+    .playlist_entry_get_end_time = playlist_entry_get_end_time,
 };
 
 /*****************************************************************/
@@ -414,7 +389,7 @@ static mowgli_dictionary_t *pvt_data_dict = NULL;
 
 static mowgli_list_t *headers_list = NULL;
 
-void plugin_set_current(Plugin *plugin)
+void plugin_set_current(Plugin * plugin)
 {
     g_static_private_set(&cur_plugin_key, plugin, NULL);
 }
@@ -427,80 +402,74 @@ static Plugin *plugin_get_current(void)
 static void set_pvt_data(Plugin * plugin, gpointer data)
 {
     mowgli_dictionary_elem_t *elem;
+    gchar *base_filename;
 
-    elem = mowgli_dictionary_find(pvt_data_dict, g_path_get_basename(plugin->filename));
+    base_filename = g_path_get_basename(plugin->filename);
+    elem = mowgli_dictionary_find(pvt_data_dict, base_filename);
     if (elem == NULL)
-        mowgli_dictionary_add(pvt_data_dict, g_path_get_basename(plugin->filename), data);
+        mowgli_dictionary_add(pvt_data_dict, base_filename, data);
     else
         elem->data = data;
+
+    g_free(base_filename);
 }
 
 static gpointer get_pvt_data(void)
 {
     Plugin *cur_p = plugin_get_current();
+    gchar *base_filename;
+    gpointer result;
 
-    return mowgli_dictionary_retrieve(pvt_data_dict, g_path_get_basename(cur_p->filename));
+    base_filename = g_path_get_basename(cur_p->filename);
+
+    result = mowgli_dictionary_retrieve(pvt_data_dict, base_filename);
+    g_free(base_filename);
+    return result;
 }
 
-static gint
-inputlist_compare_func(gconstpointer a, gconstpointer b)
+static gint inputlist_compare_func(gconstpointer a, gconstpointer b)
 {
     const InputPlugin *ap = a, *bp = b;
-    if(ap->description && bp->description)
+    if (ap->description && bp->description)
         return strcasecmp(ap->description, bp->description);
     else
         return 0;
 }
 
-static gint
-outputlist_compare_func(gconstpointer a, gconstpointer b)
+static gint outputlist_compare_func(gconstpointer a, gconstpointer b)
 {
     const OutputPlugin *ap = a, *bp = b;
     return (bp->probe_priority - ap->probe_priority);
 }
 
-static gint
-effectlist_compare_func(gconstpointer a, gconstpointer b)
+static gint effectlist_compare_func(gconstpointer a, gconstpointer b)
 {
     const EffectPlugin *ap = a, *bp = b;
-    if(ap->description && bp->description)
+    if (ap->description && bp->description)
         return strcasecmp(ap->description, bp->description);
     else
         return 0;
 }
 
-static gint
-generallist_compare_func(gconstpointer a, gconstpointer b)
+static gint generallist_compare_func(gconstpointer a, gconstpointer b)
 {
     const GeneralPlugin *ap = a, *bp = b;
-    if(ap->description && bp->description)
+    if (ap->description && bp->description)
         return strcasecmp(ap->description, bp->description);
     else
         return 0;
 }
 
-static gint
-vislist_compare_func(gconstpointer a, gconstpointer b)
+static gint vislist_compare_func(gconstpointer a, gconstpointer b)
 {
     const VisPlugin *ap = a, *bp = b;
-    if(ap->description && bp->description)
+    if (ap->description && bp->description)
         return strcasecmp(ap->description, bp->description);
     else
         return 0;
 }
 
-static gint
-discoverylist_compare_func(gconstpointer a, gconstpointer b)
-{
-    const DiscoveryPlugin *ap = a, *bp = b;
-    if(ap->description && bp->description)
-        return strcasecmp(ap->description, bp->description);
-    else
-        return 0;
-}
-
-static gboolean
-plugin_is_duplicate(const gchar * filename)
+static gboolean plugin_is_duplicate(const gchar * filename)
 {
     gchar *base_filename = g_path_get_basename(filename);
 
@@ -515,8 +484,7 @@ plugin_is_duplicate(const gchar * filename)
     return FALSE;
 }
 
-gboolean
-plugin_is_enabled(const gchar *filename)
+gboolean plugin_is_enabled(const gchar * filename)
 {
     Plugin *plugin = mowgli_dictionary_retrieve(plugin_dict, filename);
 
@@ -526,8 +494,7 @@ plugin_is_enabled(const gchar *filename)
     return plugin->enabled;
 }
 
-void
-plugin_set_enabled(const gchar *filename, gboolean enabled)
+void plugin_set_enabled(const gchar * filename, gboolean enabled)
 {
     Plugin *plugin = mowgli_dictionary_retrieve(plugin_dict, filename);
 
@@ -537,25 +504,21 @@ plugin_set_enabled(const gchar *filename, gboolean enabled)
     plugin->enabled = enabled;
 }
 
-Plugin *
-plugin_get_plugin(const gchar *filename)
+Plugin *plugin_get_plugin(const gchar * filename)
 {
     return mowgli_dictionary_retrieve(plugin_dict, filename);
 }
 
-static void
-input_plugin_init(Plugin * plugin)
+static void input_plugin_init(Plugin * plugin)
 {
     InputPlugin *p = INPUT_PLUGIN(plugin);
+    gchar *base_filename;
 
-    p->get_vis_type = input_get_vis_type;
-    p->add_vis_pcm = input_add_vis_pcm;
+    base_filename = g_path_get_basename(p->filename);
 
-    /* Pretty const casts courtesy of XMMS's plugin.h legacy. Anyone
-       else thinks we could use a CONST macro to solve the warnings?
-       - descender */
-    p->set_info = (void (*)(gchar *, gint, gint, gint, gint)) playlist_set_info_old_abi;
-    p->set_info_text = input_set_info_text;
+    /* deprecated */
+    p->set_info = ip_set_info;
+    p->set_info_text = ip_set_info_text;
 
     ip_data.input_list = g_list_append(ip_data.input_list, p);
 
@@ -563,76 +526,89 @@ input_plugin_init(Plugin * plugin)
 
     /* XXX: we need something better than p->filename if plugins
        will eventually provide multiple plugins --nenolod */
-    mowgli_dictionary_add(plugin_dict, g_path_get_basename(p->filename), p);
+    mowgli_dictionary_add(plugin_dict, base_filename, p);
+    g_free(base_filename);
 
     /* build the extension hash table */
     gint i;
-    if(p->vfs_extensions) {
-        for(i = 0; p->vfs_extensions[i] != NULL; i++) {
+    if (p->vfs_extensions)
+    {
+        for (i = 0; p->vfs_extensions[i] != NULL; i++)
+        {
             GList *hdr = NULL;
-            GList **handle = NULL; //allocated as auto in stack.
+            GList **handle = NULL;      //allocated as auto in stack.
             GList **handle2 = g_malloc(sizeof(GList **));
 
             handle = g_hash_table_lookup(ext_hash, p->vfs_extensions[i]);
-            if(handle)
+            if (handle)
                 hdr = *handle;
-            hdr = g_list_append(hdr, p); //returned hdr is non-volatile
+            hdr = g_list_append(hdr, p);        //returned hdr is non-volatile
             *handle2 = hdr;
             g_hash_table_replace(ext_hash, g_strdup(p->vfs_extensions[i]), handle2);
         }
     }
 }
 
-static void
-output_plugin_init(Plugin * plugin)
+static void output_plugin_init(Plugin * plugin)
 {
     OutputPlugin *p = OUTPUT_PLUGIN(plugin);
+    gchar *base_filename;
+
+    base_filename = g_path_get_basename(p->filename);
+
     op_data.output_list = g_list_append(op_data.output_list, p);
 
-    mowgli_dictionary_add(plugin_dict, g_path_get_basename(p->filename), p);
+    mowgli_dictionary_add(plugin_dict, base_filename, p);
+
+    g_free(base_filename);
 }
 
-static void
-effect_plugin_init(Plugin * plugin)
+static void effect_plugin_init(Plugin * plugin)
 {
     EffectPlugin *p = EFFECT_PLUGIN(plugin);
+    gchar *base_filename;
+
+    base_filename = g_path_get_basename(p->filename);
+
     ep_data.effect_list = g_list_append(ep_data.effect_list, p);
 
-    mowgli_dictionary_add(plugin_dict, g_path_get_basename(p->filename), p);
+    mowgli_dictionary_add(plugin_dict, base_filename, p);
+
+    g_free(base_filename);
 }
 
-static void
-general_plugin_init(Plugin * plugin)
+static void general_plugin_init(Plugin * plugin)
 {
     GeneralPlugin *p = GENERAL_PLUGIN(plugin);
+    gchar *base_filename;
+
+    base_filename = g_path_get_basename(plugin->filename);
+
     gp_data.general_list = g_list_append(gp_data.general_list, p);
 
-    mowgli_dictionary_add(plugin_dict, g_path_get_basename(p->filename), p);
+    mowgli_dictionary_add(plugin_dict, base_filename, p);
+
+    g_free(base_filename);
 }
 
-static void
-vis_plugin_init(Plugin * plugin)
+static void vis_plugin_init(Plugin * plugin)
 {
     VisPlugin *p = VIS_PLUGIN(plugin);
+    gchar *base_filename;
+
+    base_filename = g_path_get_basename(plugin->filename);
+
     p->disable_plugin = vis_disable_plugin;
     vp_data.vis_list = g_list_append(vp_data.vis_list, p);
 
-    mowgli_dictionary_add(plugin_dict, g_path_get_basename(p->filename), p);
-}
+    mowgli_dictionary_add(plugin_dict, base_filename, p);
 
-static void
-discovery_plugin_init(Plugin * plugin)
-{
-    DiscoveryPlugin *p = DISCOVERY_PLUGIN(plugin);
-    dp_data.discovery_list = g_list_append(dp_data.discovery_list, p);
-
-    mowgli_dictionary_add(plugin_dict, g_path_get_basename(p->filename), p);
+    g_free(base_filename);
 }
 
 /*******************************************************************/
 
-static void
-plugin2_dispose(GModule *module, const gchar *str, ...)
+static void plugin2_dispose(GModule * module, const gchar * str, ...)
 {
     gchar *buf;
     va_list va;
@@ -647,8 +623,7 @@ plugin2_dispose(GModule *module, const gchar *str, ...)
     g_module_close(module);
 }
 
-void
-plugin2_process(PluginHeader *header, GModule *module, const gchar *filename)
+void plugin2_process(PluginHeader * header, GModule * module, const gchar * filename)
 {
     gint i, n;
     mowgli_node_t *hlist_node;
@@ -657,8 +632,7 @@ plugin2_process(PluginHeader *header, GModule *module, const gchar *filename)
         return plugin2_dispose(module, "plugin <%s> discarded, invalid module magic", filename);
 
     if (header->api_version != __AUDACIOUS_PLUGIN_API__)
-        return plugin2_dispose(module, "plugin <%s> discarded, wanting API version %d, we implement API version %d",
-                               filename, header->api_version, __AUDACIOUS_PLUGIN_API__);
+        return plugin2_dispose(module, "plugin <%s> discarded, wanting API version %d, we implement API version %d", filename, header->api_version, __AUDACIOUS_PLUGIN_API__);
 
     hlist_node = mowgli_node_create();
     mowgli_node_add(header, hlist_node, headers_list);
@@ -718,21 +692,11 @@ plugin2_process(PluginHeader *header, GModule *module, const gchar *filename)
         }
     }
 
-    if (header->dp_list)
-    {
-        for (i = 0; (header->dp_list)[i] != NULL; i++, n++)
-        {
-            PLUGIN((header->dp_list)[i])->filename = g_strdup_printf("%s (#%d)", filename, n);
-            discovery_plugin_init(PLUGIN((header->dp_list)[i]));
-        }
-    }
-
     if (header->interface)
         interface_register(header->interface);
 }
 
-void
-plugin2_unload(PluginHeader *header, mowgli_node_t *hlist_node)
+void plugin2_unload(PluginHeader * header, mowgli_node_t * hlist_node)
 {
     GModule *module;
 
@@ -757,8 +721,7 @@ plugin2_unload(PluginHeader *header, mowgli_node_t *hlist_node)
 
 /******************************************************************/
 
-static void
-add_plugin(const gchar * filename)
+static void add_plugin(const gchar * filename)
 {
     GModule *module;
     gpointer func;
@@ -768,16 +731,16 @@ add_plugin(const gchar * filename)
 
     g_message("Loaded plugin (%s)", filename);
 
-    if (!(module = g_module_open(filename, G_MODULE_BIND_LOCAL))) {
-        printf("Failed to load plugin (%s): %s\n",
-                  filename, g_module_error());
+    if (!(module = g_module_open(filename, G_MODULE_BIND_LAZY | G_MODULE_BIND_LOCAL)))
+    {
+        printf("Failed to load plugin (%s): %s\n", filename, g_module_error());
         return;
     }
 
     /* v2 plugin loading */
     if (g_module_symbol(module, "get_plugin_info", &func))
     {
-        PluginHeader *(*header_func_p)(struct _AudaciousFuncTableV1 *) = func;
+        PluginHeader *(*header_func_p) (struct _AudaciousFuncTableV1 *) = func;
         PluginHeader *header;
 
         /* this should never happen. */
@@ -791,8 +754,7 @@ add_plugin(const gchar * filename)
     g_module_close(module);
 }
 
-static gboolean
-scan_plugin_func(const gchar * path, const gchar * basename, gpointer data)
+static gboolean scan_plugin_func(const gchar * path, const gchar * basename, gpointer data)
 {
     if (!str_has_suffix_nocase(basename, SHARED_SUFFIX))
         return FALSE;
@@ -805,34 +767,27 @@ scan_plugin_func(const gchar * path, const gchar * basename, gpointer data)
     return FALSE;
 }
 
-static void
-scan_plugins(const gchar * path)
+static void scan_plugins(const gchar * path)
 {
     dir_foreach(path, scan_plugin_func, NULL, NULL);
 }
 
-void
-plugin_system_init(void)
+void plugin_system_init(void)
 {
     gchar *dir, **disabled;
     GList *node;
     OutputPlugin *op;
     InputPlugin *ip;
     LowlevelPlugin *lp;
-    DiscoveryPlugin *dp;
     GtkWidget *dialog;
     gint dirsel = 0, i = 0;
     gint prio;
 
-    if (!g_module_supported()) {
-        dialog =
-            gtk_message_dialog_new (NULL,
-                                    GTK_DIALOG_DESTROY_WITH_PARENT,
-                                    GTK_MESSAGE_ERROR,
-                                    GTK_BUTTONS_CLOSE,
-                                    _("Module loading not supported! Plugins will not be loaded.\n"));
-        gtk_dialog_run (GTK_DIALOG (dialog));
-        gtk_widget_destroy (dialog);
+    if (!g_module_supported())
+    {
+        dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, _("Module loading not supported! Plugins will not be loaded.\n"));
+        gtk_dialog_run(GTK_DIALOG(dialog));
+        gtk_widget_destroy(dialog);
         return;
     }
 
@@ -847,29 +802,29 @@ plugin_system_init(void)
 #ifndef DISABLE_USER_PLUGIN_DIR
     scan_plugins(aud_paths[BMP_PATH_USER_PLUGIN_DIR]);
     /*
-     * This is in a separate lo
-     * DiscoveryPlugin *dpop so if the user puts them in the
+     * This is in a separate loop so if the user puts them in the
      * wrong dir we'll still get them in the right order (home dir
      * first)                                                - Zinx
      */
-    while (plugin_dir_list[dirsel]) {
-        dir = g_build_filename(aud_paths[BMP_PATH_USER_PLUGIN_DIR],
-                               plugin_dir_list[dirsel++], NULL);
+    while (plugin_dir_list[dirsel])
+    {
+        dir = g_build_filename(aud_paths[BMP_PATH_USER_PLUGIN_DIR], plugin_dir_list[dirsel++], NULL);
         scan_plugins(dir);
         g_free(dir);
     }
     dirsel = 0;
 #endif
 
-    while (plugin_dir_list[dirsel]) {
+    while (plugin_dir_list[dirsel])
+    {
         dir = g_build_filename(PLUGIN_DIR, plugin_dir_list[dirsel++], NULL);
         scan_plugins(dir);
         g_free(dir);
     }
 
     op_data.output_list = g_list_sort(op_data.output_list, outputlist_compare_func);
-    if (!op_data.current_output_plugin
-        && g_list_length(op_data.output_list)) {
+    if (!op_data.current_output_plugin && g_list_length(op_data.output_list))
+    {
         op_data.current_output_plugin = op_data.output_list->data;
     }
 
@@ -884,14 +839,9 @@ plugin_system_init(void)
     vp_data.vis_list = g_list_sort(vp_data.vis_list, vislist_compare_func);
     vp_data.enabled_list = NULL;
 
-    dp_data.discovery_list = g_list_sort(dp_data.discovery_list, discoverylist_compare_func);
-    dp_data.enabled_list = NULL;
-
-
     general_enable_from_stringified_list(cfg.enabled_gplugins);
     vis_enable_from_stringified_list(cfg.enabled_vplugins);
     effect_enable_from_stringified_list(cfg.enabled_eplugins);
-    discovery_enable_from_stringified_list(cfg.enabled_dplugins);
 
     g_free(cfg.enabled_gplugins);
     cfg.enabled_gplugins = NULL;
@@ -905,91 +855,97 @@ plugin_system_init(void)
     g_free(cfg.enabled_dplugins);
     cfg.enabled_dplugins = NULL;
 
-    if (!cfg.outputplugin) {
-        for (prio = 10; prio >= 0; prio--) {
-            for (node = op_data.output_list; node; node = g_list_next(node)) {
+    if (!cfg.outputplugin)
+    {
+        for (prio = 10; prio >= 0; prio--)
+        {
+            for (node = op_data.output_list; node; node = g_list_next(node))
+            {
                 op = OUTPUT_PLUGIN(node->data);
 
                 if (op->probe_priority != prio)
                     continue;
 
-		if (op->init)
-		{
-		    OutputPluginInitStatus ret = op->init();
-		    if (ret == OUTPUT_PLUGIN_INIT_NO_DEVICES)
-		    {
-		        g_message("Plugin %s reports no devices. Attempting to avert disaster, trying others.\n",
-                                  g_path_get_basename(op->filename));
-                    }
-                    else if (ret == OUTPUT_PLUGIN_INIT_FAIL)
-                    {
-		        g_message("Plugin %s was unable to initialise. Attemping to avert disaster, trying others.\n",
-		                  g_path_get_basename(op->filename));
-                    }
-                    else if (ret == OUTPUT_PLUGIN_INIT_FOUND_DEVICES)
-                    {
+                if (op->init)
+                {
+                    OutputPluginInitStatus ret = op->init();
+                    gchar *base_filename = g_path_get_basename(op->filename);
+                    switch (ret) {
+                    case OUTPUT_PLUGIN_INIT_NO_DEVICES:
+                        g_message("Plugin %s reports no devices. Attempting to avert disaster, trying others.",
+                            base_filename);
+                        break;
+                    case OUTPUT_PLUGIN_INIT_FAIL:
+                        g_message("Plugin %s was unable to initialise. Attemping to avert disaster, trying others.",
+                            base_filename);
+                        break;
+                    case OUTPUT_PLUGIN_INIT_FOUND_DEVICES:
+                        g_message("Plugin %s found devices.", base_filename);
                         if (!op_data.current_output_plugin)
                             op_data.current_output_plugin = op;
-                    }
-                    else if (!op_data.current_output_plugin)
-                    {
-		        g_message("Plugin %s did not report status, and no plugin has worked yet. Do you still need to convert it? Selecting for now...\n",
-		                  g_path_get_basename(op->filename));
-
+                        break;
+                    default:
+                        g_message("Plugin %s did not report status, and no plugin has worked yet. "
+                            "Do you still need to convert it? Selecting for now...",
+                            base_filename);
                         if (!op_data.current_output_plugin)
                             op_data.current_output_plugin = op;
+                        break;
                     }
-		}
+                    g_free(base_filename);
+                }
             }
         }
     }
     else
     {
-        for (node = op_data.output_list; node; node = g_list_next(node)) {
+        for (node = op_data.output_list; node; node = g_list_next(node))
+        {
+            gchar *plugin_base_filename;
+            gchar *op_base_filename;
             op = OUTPUT_PLUGIN(node->data);
 
-            if (op->init) {
-                plugin_set_current((Plugin *)op);
+            if (op->init)
+            {
+                plugin_set_current((Plugin *) op);
                 op->init();
             }
 
-            if (!g_ascii_strcasecmp(g_path_get_basename(cfg.outputplugin), g_path_get_basename(op->filename)))
+            plugin_base_filename = g_path_get_basename(cfg.outputplugin);
+            op_base_filename = g_path_get_basename(op->filename);
+            if (!g_ascii_strcasecmp(plugin_base_filename, op_base_filename))
                 op_data.current_output_plugin = op;
+            g_free(plugin_base_filename);
+            g_free(op_base_filename);
         }
     }
 
-    for (node = ip_data.input_list; node; node = g_list_next(node)) {
+    for (node = ip_data.input_list; node; node = g_list_next(node))
+    {
         ip = INPUT_PLUGIN(node->data);
         if (ip->init)
-	{
-	    plugin_set_current((Plugin *)ip);
+        {
+            plugin_set_current((Plugin *) ip);
             ip->init();
-	}
+        }
     }
 
-    for (node = dp_data.discovery_list; node; node = g_list_next(node)) {
-        dp = DISCOVERY_PLUGIN(node->data);
-        if (dp->init)
-	{
-	    plugin_set_current((Plugin *)dp);
-            dp->init();
-	}
-    }
-
-
-    for (node = lowlevel_list; node; node = g_list_next(node)) {
+    for (node = lowlevel_list; node; node = g_list_next(node))
+    {
         lp = LOWLEVEL_PLUGIN(node->data);
         if (lp->init)
-	{
-	    plugin_set_current((Plugin *)lp);
+        {
+            plugin_set_current((Plugin *) lp);
             lp->init();
-	}
+        }
     }
 
-    if (cfg.disabled_iplugins) {
+    if (cfg.disabled_iplugins)
+    {
         disabled = g_strsplit(cfg.disabled_iplugins, ":", 0);
 
-        while (disabled[i]) {
+        while (disabled[i])
+        {
             Plugin *plugintmp = plugin_get_plugin(disabled[i]);
             g_free(disabled[i]);
             if (plugintmp)
@@ -1004,14 +960,12 @@ plugin_system_init(void)
     }
 }
 
-static void
-remove_list(gpointer key, gpointer value, gpointer data)
+static void remove_list(gpointer key, gpointer value, gpointer data)
 {
-    g_list_free(*(GList **)value);
+    g_list_free(*(GList **) value);
 }
 
-void
-plugin_system_cleanup(void)
+void plugin_system_cleanup(void)
 {
     InputPlugin *ip;
     OutputPlugin *op;
@@ -1019,13 +973,13 @@ plugin_system_cleanup(void)
     GeneralPlugin *gp;
     VisPlugin *vp;
     LowlevelPlugin *lp;
-    DiscoveryPlugin *dp;
     GList *node;
     mowgli_node_t *hlist_node;
 
     g_message("Shutting down plugin system");
 
-    if (playback_get_playing()) {
+    if (playback_get_playing())
+    {
         ip_data.stop = TRUE;
         playback_stop();
         ip_data.stop = FALSE;
@@ -1034,10 +988,12 @@ plugin_system_cleanup(void)
     /* FIXME: race condition -nenolod */
     op_data.current_output_plugin = NULL;
 
-    for (node = get_input_list(); node; node = g_list_next(node)) {
+    for (node = get_input_list(); node; node = g_list_next(node))
+    {
         ip = INPUT_PLUGIN(node->data);
-        if (ip) {
-	    plugin_set_current((Plugin *)ip);
+        if (ip)
+        {
+            plugin_set_current((Plugin *) ip);
 
             if (ip->cleanup)
                 ip->cleanup();
@@ -1054,10 +1010,12 @@ plugin_system_cleanup(void)
         ip_data.input_list = NULL;
     }
 
-    for (node = get_output_list(); node; node = g_list_next(node)) {
+    for (node = get_output_list(); node; node = g_list_next(node))
+    {
         op = OUTPUT_PLUGIN(node->data);
-        if (op) {
-	    plugin_set_current((Plugin *)op);
+        if (op)
+        {
+            plugin_set_current((Plugin *) op);
 
             if (op->cleanup)
                 op->cleanup();
@@ -1074,10 +1032,12 @@ plugin_system_cleanup(void)
         op_data.output_list = NULL;
     }
 
-    for (node = get_effect_list(); node; node = g_list_next(node)) {
+    for (node = get_effect_list(); node; node = g_list_next(node))
+    {
         ep = EFFECT_PLUGIN(node->data);
-        if (ep) {
-	    plugin_set_current((Plugin *)ep);
+        if (ep)
+        {
+            plugin_set_current((Plugin *) ep);
 
             if (ep->cleanup)
                 ep->cleanup();
@@ -1094,10 +1054,12 @@ plugin_system_cleanup(void)
         ep_data.effect_list = NULL;
     }
 
-    for (node = get_general_list(); node; node = g_list_next(node)) {
+    for (node = get_general_list(); node; node = g_list_next(node))
+    {
         gp = GENERAL_PLUGIN(node->data);
-        if (gp) {
-	    plugin_set_current((Plugin *)gp);
+        if (gp)
+        {
+            plugin_set_current((Plugin *) gp);
 
             if (gp->cleanup)
                 gp->cleanup();
@@ -1114,10 +1076,12 @@ plugin_system_cleanup(void)
         gp_data.general_list = NULL;
     }
 
-    for (node = get_vis_list(); node; node = g_list_next(node)) {
+    for (node = get_vis_list(); node; node = g_list_next(node))
+    {
         vp = VIS_PLUGIN(node->data);
-        if (vp) {
-	    plugin_set_current((Plugin *)vp);
+        if (vp)
+        {
+            plugin_set_current((Plugin *) vp);
 
             if (vp->cleanup)
                 vp->cleanup();
@@ -1134,31 +1098,12 @@ plugin_system_cleanup(void)
         vp_data.vis_list = NULL;
     }
 
-
-    for (node = get_discovery_list(); node; node = g_list_next(node)) {
-        dp = DISCOVERY_PLUGIN(node->data);
-        if (dp) {
-	    plugin_set_current((Plugin *)dp);
-
-            if (dp->cleanup)
-                dp->cleanup();
-
-            GDK_THREADS_LEAVE();
-            while (g_main_context_iteration(NULL, FALSE));
-            GDK_THREADS_ENTER();
-        }
-    }
-
-    if (dp_data.discovery_list != NULL)
+    for (node = lowlevel_list; node; node = g_list_next(node))
     {
-        g_list_free(dp_data.discovery_list);
-        dp_data.discovery_list = NULL;
-    }
-
-    for (node = lowlevel_list; node; node = g_list_next(node)) {
         lp = LOWLEVEL_PLUGIN(node->data);
-        if (lp) {
-	    plugin_set_current((Plugin *)lp);
+        if (lp)
+        {
+            plugin_set_current((Plugin *) lp);
 
             if (lp->cleanup)
                 lp->cleanup();
@@ -1182,8 +1127,7 @@ plugin_system_cleanup(void)
         vfs_transports = NULL;
     }
 
-    MOWGLI_LIST_FOREACH(hlist_node, headers_list->head)
-    	plugin2_unload(hlist_node->data, hlist_node);
+    MOWGLI_LIST_FOREACH(hlist_node, headers_list->head) plugin2_unload(hlist_node->data, hlist_node);
 
     mowgli_dictionary_destroy(plugin_dict, NULL, NULL);
     mowgli_dictionary_destroy(pvt_data_dict, NULL, NULL);
