@@ -28,10 +28,12 @@
 #include <gtk/gtk.h>
 #include <string.h>
 
+#include "compatibility.h"
+
 #include "main.h"
-#include "playlist.h"
 #include "playback.h"
-#include "strings.h"
+#include "playlist-new.h"
+#include "audstrings.h"
 #include "ui_fileinfopopup.h"
 #include "ui_fileinfo.h"
 
@@ -75,7 +77,7 @@ filepopup_entry_set_image(GtkWidget *filepopup_win, const gchar *entry_name,
             height = cfg.filepopup_pixelsize;
             width = (int)(cfg.filepopup_pixelsize / aspect);
         }
-        
+
         pixbuf2 = gdk_pixbuf_scale_simple(GDK_PIXBUF(pixbuf), width, height,
                                           GDK_INTERP_BILINEAR);
         g_object_unref(G_OBJECT(pixbuf));
@@ -92,22 +94,22 @@ fileinfopopup_progress_cb(gpointer filepopup_win)
     GtkWidget *progressbar =
         g_object_get_data(G_OBJECT(filepopup_win), "progressbar");
     gchar *tooltip_file = g_object_get_data(G_OBJECT(filepopup_win), "file");
-    gchar *current_file_u, *current_file;
-    Playlist *pl;
+    gchar * current_file;
     gint length =
         GPOINTER_TO_INT(g_object_get_data(G_OBJECT(filepopup_win), "length"));
-    gint pos, time;
+    gint playlist, entry, time;
+    const gchar * filename;
 
     g_return_val_if_fail(progressbar != NULL, FALSE);
 
-    pl = playlist_get_active();
-    g_return_val_if_fail(pl != NULL, FALSE);
+    playlist = playlist_get_active ();
+    entry = playlist_get_position (playlist);
+    filename = playlist_entry_get_filename (playlist, entry);
 
-    pos = playlist_get_position(pl);
+    if (filename == NULL)
+        return FALSE;
 
-    current_file_u = playlist_get_filename(pl , pos);
-    g_return_val_if_fail(current_file_u != NULL, FALSE);
-    current_file = g_filename_from_uri(current_file_u, NULL, NULL);
+    current_file = g_filename_from_uri (filename, NULL, NULL);
 
     if (playback_get_playing() && length != -1 &&
         current_file != NULL && tooltip_file != NULL &&
@@ -189,7 +191,7 @@ fileinfopopup_add_category(GtkWidget *filepopup_win,
     gtk_misc_set_alignment(GTK_MISC(filepopup_data_info_label), 0, 0.5);
     gtk_misc_set_padding(GTK_MISC(filepopup_data_info_header), 0, 3);
     gtk_misc_set_padding(GTK_MISC(filepopup_data_info_label), 0, 3);
-    
+
     markup =
         g_markup_printf_escaped("<span style=\"italic\">%s</span>", category);
 
@@ -224,8 +226,6 @@ fileinfopopup_create(void)
 		    		 GDK_WINDOW_TYPE_HINT_TOOLTIP);
     gtk_window_set_decorated(GTK_WINDOW(filepopup_win), FALSE);
     gtk_container_set_border_width(GTK_CONTAINER(filepopup_win), 6);
-    gtk_window_set_transient_for(GTK_WINDOW(filepopup_win),
-                                 GTK_WINDOW(mainwin));
 
     filepopup_hbox = gtk_hbox_new(FALSE, 0);
     gtk_container_add(GTK_CONTAINER(filepopup_win), filepopup_hbox);
@@ -273,7 +273,7 @@ fileinfopopup_create(void)
                                          _("Track Length"),
                                          "header_tracklen", "label_tracklen",
                                          6);
-    
+
     gtk_table_set_row_spacing(GTK_TABLE(filepopup_data_table), 6, 6);
 
     /* track progress */
@@ -287,7 +287,7 @@ fileinfopopup_create(void)
     g_object_set_data(G_OBJECT(filepopup_win), "progressbar",
                       filepopup_progress);
     fileinfopopup_progress_init(filepopup_win);
-    
+
     /* this will realize all widgets contained in filepopup_hbox */
     gtk_widget_show_all(filepopup_hbox);
 
@@ -354,7 +354,7 @@ fileinfopopup_show_from_tuple(GtkWidget *filepopup_win,
     }
     if (tuple_get_string(tuple, FIELD_FILE_PATH, NULL) && tuple_get_string(tuple, FIELD_FILE_NAME, NULL))
         g_object_set_data(G_OBJECT(filepopup_win), "file",
-                          g_build_filename(tuple_get_string(tuple, FIELD_FILE_PATH, NULL), 
+                          g_build_filename(tuple_get_string(tuple, FIELD_FILE_PATH, NULL),
                                            tuple_get_string(tuple, FIELD_FILE_NAME, NULL),
                                            NULL));
 
@@ -373,11 +373,9 @@ fileinfopopup_show_from_tuple(GtkWidget *filepopup_win,
         /* display filename if track_name is not available */
         gchar *markup =
             g_markup_printf_escaped("<span style=\"italic\">%s</span>", _("Filename"));
-        gchar *utf_filename = filename_to_utf8(tuple_get_string(tuple, FIELD_FILE_NAME, NULL));
         gtk_label_set_markup(GTK_LABEL(g_object_get_data(G_OBJECT(filepopup_win), "header_title")), markup);
         g_free(markup);
-        filepopup_entry_set_text(filepopup_win, "label_title", utf_filename);
-        g_free(utf_filename);
+        filepopup_entry_set_text(filepopup_win, "label_title", tuple_get_string(tuple, FIELD_FILE_NAME, NULL));
     }
 
     fileinfopupup_update_data(filepopup_win, tuple_get_string(tuple, FIELD_ARTIST, NULL),
@@ -408,7 +406,7 @@ fileinfopopup_show_from_tuple(GtkWidget *filepopup_win,
     fileinfopupup_update_data(filepopup_win, track_string,
                                         "label_tracknum", "header_tracknum");
     g_free(track_string);
-    
+
     if (tuple_get_string(tuple, FIELD_FILE_NAME, NULL) && tuple_get_string(tuple, FIELD_FILE_PATH, NULL)) {
         tmp = fileinfo_recursive_get_image(tuple_get_string(tuple, FIELD_FILE_PATH, NULL), tuple_get_string(tuple, FIELD_FILE_NAME, NULL), 0);
         if (tmp) { // picture found
@@ -436,7 +434,7 @@ fileinfopopup_show_from_tuple(GtkWidget *filepopup_win,
     /* start a timer that updates a progress bar if the tooltip
        is shown for the song that is being currently played */
     if (fileinfopopup_progress_check_active(filepopup_win) == FALSE)
-    { 
+    {
         fileinfopopup_progress_start(filepopup_win);
         /* immediately run the callback once to update progressbar status */
         fileinfopopup_progress_cb(filepopup_win);
